"""
A spin-off experiment may start from a day, where no event-generator restart file is available.
This script finds the last available restart file in the event-generator directory.
From this date, the event-generator member can be restarted.
"""
from glob import glob
from dateutil.parser import parse
from datetime import datetime, timedelta
from pathlib import Path
import ast
import os

# Get some autosubmit variables
WORKDIR = "%HPCROOTDIR%"
STARTDATE = "%SDATE%"
# Get further autosubmit variables
CURRENT_DATE = "%SDATE%"
DATELIST_STRING = "%experiment.DATELIST%"
FROM_MEMBERS_STRING = "%experiment.FROM_MEMBERS%"
NOT_MORE_THAN_N_DAYS_BACK = "%experiment.RSFILE_MAXDAYS_BACK%"

# parse FROM_MEMBERS and DATELIST into a python list
DATELIST = ast.literal_eval(DATELIST_STRING)
FROM_MEMBERS = ast.literal_eval(FROM_MEMBERS_STRING)


# get index of current start date in datelist and therefrom infer from_member
def find_index_in_list(number_list, target_number):
    try:
        index = number_list.index(target_number)
        return index
    except ValueError:
        return -1


index = find_index_in_list(DATELIST, CURRENT_DATE)
if index != -1:
    print(f"The date {CURRENT_DATE} is at index {index} in the datelist.")
else:
    print(f"The date {CURRENT_DATE} is not in the datelist.")
FROM_MEMBER = FROM_MEMBERS[index]

# determine parent directory for restart
PARENTDIR = "%initial_conditions.parent_folder%" + f"m{FROM_MEMBER:0d}/"
# create target directory for restart parent
RUNDIR = f"{WORKDIR}/{STARTDATE}_from_parent_{FROM_MEMBER}"
try:
    os.makedirs(RUNDIR)
    print(f"Directory '{RUNDIR}' created successfully.")
except OSError as e:
    print(f"An error occurred: {e}")


def infer_restart_dates_from_files_in_direcory(directory):
    restart_files = glob(directory + '/*restart*.nc')

    inferred_restart_file_dates = [parse(f.split("atm")[-1], fuzzy=True, ignoretz=True) for f in restart_files]
    return inferred_restart_file_dates


def find_date_before(target_date, dates_list, max_lag=None):
    # Convert the target_date to a datetime object if it's not already
    if not isinstance(target_date, datetime):
        target_date = datetime.strptime(target_date, '%Y%m%d')  # Adjust the format if needed

    # Calculate the minimum date allowed (if NOT_MORE_THAN_N_DAYS is provided)
    if max_lag is not None:
        min_date_allowed = target_date - timedelta(days=max_lag)
    else:
        min_date_allowed = datetime.min  # If NOT_MORE_THAN_N_DAYS is not provided, set a very early date

    # Filter dates_list to get only the dates that occur before the target_date
    filtered_dates = [dt for dt in dates_list if min_date_allowed <= dt < target_date]

    # Find the maximum date from the filtered dates (i.e., the date that occurs first before the target_date)
    if filtered_dates:
        return max(filtered_dates)
    else:
        return None  # No date found before the target_date


def link_restart_files(from_restart_date, from_directoy, into_directory):
    # sample restart files:
    # chunks/20201001/m1/icon_grid_0010_R02B04_G_restart_atm_20201016T000000Z.nc
    date_string = from_restart_date.strftime("%Y%m%dT00")
    pattern = f"{from_directoy}/*restart*{date_string}*.nc"
    files = glob(pattern)
    if len(files) == 0:
        raise AssertionError(f"No restart files match pattern {pattern}.")
    elif len(files) > 1:
        raise AssertionError(f"More than 1 file matches pattern {pattern}: {files}.")
    else:
        restart_file = files[0]

    # Path to the source file
    source_file_path = restart_file
    # Path to the directory where to create the symlink
    symlink_directory = into_directory
    # Name of the symlink (same as original)
    symlink_name = source_file_path.split('/')[-1]
    # Create the symbolic link
    symlink_path = os.path.join(symlink_directory, symlink_name)
    os.symlink(source_file_path, symlink_path)

    print(f"Symbolic link '{symlink_name}' created at '{symlink_directory}'")


if __name__ == "__main__":
    # find available restart files in event generator run
    restart_file_dates = infer_restart_dates_from_files_in_direcory(PARENTDIR)
    # find appropriate restart file prior to spin-off start date
    restart_from = find_date_before(CURRENT_DATE, restart_file_dates, max_lag=NOT_MORE_THAN_N_DAYS_BACK)
    print(f"Restarting parent run on {restart_from}.")
    # link appropriate restart files into into parental re-run directory
    link_restart_files(from_restart_date=restart_from, from_directoy=PARENTDIR, into_directory=RUNDIR)
